# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2018 Jennifer Wu
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
from pyswmm.swmm5 import PYSWMMException
from pyswmm.toolkitapi import ObjectType
from pyswmm.lidlayers import Surface, Soil, Storage, Pavement, Drain, DrainMat


class LidControls(object):
    """
    Lid Control Iterator Methods.

    :param object model: Open Model Instance
    """

    def __init__(self, model):
        if not model._model.fileLoaded:
            raise PYSWMMException("SWMM Model Not Open")
        self._sim = model
        self._model = model._model
        self._cuindex = 0
        self._nlidcontrols = self._model.getProjectSize(ObjectType.LID.value)

    def __len__(self):
        """
        Return number of lid controls.

        Use the expression 'len(LidControls)'.

        :return: Number of Lid Controls
        :rtype: int

        """
        return self._model.getProjectSize(ObjectType.LID.value)

    def __contains__(self, lidcontrolid):
        """
        Checks if Lid Control ID exists.

        :return: ID Exists
        :rtype: bool
        """
        return self._model.ObjectIDexist(ObjectType.LID.value,
                                         lidcontrolid)

    def __getitem__(self, lidcontrolid):
        if self.__contains__(lidcontrolid):
            return LidControl(self._sim, self._model, lidcontrolid)
        else:
            raise PYSWMMException("Lid Control ID Does not Exist")

    def __iter__(self):
        return self

    def __next__(self):
        if self._cuindex < self._nlidcontrols:
            lidcontrolobject = self.__getitem__(self._lidcontrolid)
            self._cuindex += 1  # Next Iteration
            return lidcontrolobject
        else:
            raise StopIteration()

    next = __next__  # Python 2

    @property
    def _lidcontrolid(self):
        """Lid Control ID."""
        return self._model.getObjectId(ObjectType.LID.value,
                                       self._cuindex)


class LidControl(object):
    def __init__(self, sim, model, lidcontrolid):
        if not model.fileLoaded:
            raise PYSWMMException("SWMM Model Not Open")
        if lidcontrolid not in model.getObjectIDList(ObjectType.LID.value):
            raise PYSWMMException("ID Not valid")
        self._sim = sim
        self._model = model
        self._lidcontrolid = lidcontrolid

        self.surface = Surface(model, self)
        self.soil = Soil(model, self)
        self.storage = Storage(model, self)
        self.pavement = Pavement(model, self)
        self.drain = Drain(model, self)
        self.drain_mat = DrainMat(model, self)

    def __str__(self):
        return self._lidcontrolid

    @property
    def can_overflow(self):
        """
        Get lid control surface layer option for immediate outflow of excess water

        :return: Parameter Value
        :rtype: char
        """
        return self._model.getLidCOverflow(self._lidcontrolid)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Standard library imports
import pytest
from random import randint
import sys

# Local imports
import pyswmm.toolkitapi as tka
from pyswmm import Links, Nodes, Simulation
from pyswmm.tests.data import MODEL_WEIR_SETTING_PATH
from pyswmm.swmm5 import SWMMException


def test_simulation_1():
    sim = Simulation(MODEL_WEIR_SETTING_PATH)
    print(f'system units: {sim.system_units}')
    print(f'swmm version: {sim._model.swmm_getVersion()}')
    allow_ponding = sim._model.getSimOptionSetting(tka.SimAnalysisSettings.AllowPonding)
    routing_step = sim._model.getSimAnalysisSetting(tka.SimulationParameters.RouteStep)
    print(f'analysis setting: {allow_ponding}')
    print(f'analysis param: {routing_step}')
    assert sim.system_units == 'US'
    assert not allow_ponding
    assert routing_step == 1
    for ind, step in enumerate(sim):
        print(sim.current_time)
        sim.step_advance(randint(300, 900))

    sim.close()


def test_simulation_2():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        for ind, step in enumerate(sim):
            print(sim.current_time)
            sim.step_advance(randint(300, 900))


def test_simulation_3():
    sim = Simulation(MODEL_WEIR_SETTING_PATH)
    sim.execute()


def test_simulation_4():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        for ind, step in enumerate(sim):
            print(sim.percent_complete)
            sim.step_advance(randint(300, 900))


def test_simulation_iter():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        c1c2 = Links(sim)["C1:C2"]

        sim.step_advance(300)
        for ind, step in enumerate(sim):
            print(c1c2.flow, c1c2.target_setting)
            if c1c2.flow > 9.19:
                c1c2.target_setting = 0.9


def test_simulation_9():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        J1 = Nodes(sim)["J1"]

        def init_function():
            J1.initial_depth = 15

        sim.initial_conditions(init_function)
        for ind, step in enumerate(sim):
            if ind == 0:
                assert J1.depth > 14


def test_simulation_callback_1():
    LIST = []
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        LIST.append("OPENED")

        def before_start1():
            LIST.append("before_start1")

        sim.add_before_start(before_start1)

        def before_step1():
            if "before_step1" not in LIST:
                LIST.append("before_step1")

        sim.add_before_step(before_step1)

        def after_step1():
            if "after_step1" not in LIST:
                LIST.append("after_step1")

        sim.add_after_step(after_step1)

        def before_end1():
            if "before_end1" not in LIST:
                LIST.append("before_end1")

        sim.add_before_end(before_end1)

        def after_end1():
            if "after_end1" not in LIST:
                LIST.append("after_end1")

        sim.add_after_end(after_end1)

        def after_close1():
            if "after_close1" not in LIST:
                LIST.append("after_close1")

        sim.add_after_close(after_close1)

        for ind, step in enumerate(sim):
            if ind == 0:
                LIST.append("SIM_STEP")

    assert LIST == [
        "OPENED", "before_start1", "before_step1", "after_step1", "SIM_STEP",
        "before_end1", "after_end1", "after_close1"
    ]
    print(LIST)


def test_simulation_terminate():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        i = 0
        for ind, step in enumerate(sim):
            i += 1
            if ind == 10:
                sim.terminate_simulation()
        assert(i == 11)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2017 Katherine M. Ratliff
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Local imports
from pyswmm import Simulation, Subcatchments, Links, Nodes
# from pyswmm.swmm5 import PySWMM
from pyswmm.tests.data import MODEL_POLLUTANTS_PATH


def test_pollutants_1():
    with Simulation(MODEL_POLLUTANTS_PATH) as sim:
        S1 = Subcatchments(sim)["S1"]
        S2 = Subcatchments(sim)["S2"]
        S3 = Subcatchments(sim)["S3"]

        C1 = Links(sim)["C1"]
        C2 = Links(sim)["C2"]
        C3 = Links(sim)["C3"]
        C4 = Links(sim)["C4"]

        J1 = Nodes(sim)["J1"]
        J2 = Nodes(sim)["J2"]
        J3 = Nodes(sim)["J3"]
        J4 = Nodes(sim)["J4"]

        for step in sim:
            pass
            # print(S1.buildup)
            # print(S2.buildup)
            # print(S3.buildup)

            # print(S1.conc_ponded)
            # print(S2.conc_ponded)
            # print(S3.conc_ponded)

            # print(S1.pollut_quality)
            # print(S2.pollut_quality)
            # print(S3.pollut_quality)

            # print(S1.runoff_total_loading)
            # print(S2.runoff_total_loading)
            # print(S3.runoff_total_loading)

            # print(C1.pollut_quality)
            # print(C2.pollut_quality)
            # print(C3.pollut_quality)
            # print(C4.pollut_quality)

            # print(C1.total_loading)
            # print(C2.total_loading)
            # print(C3.total_loading)
            # print(C4.total_loading)

            # print(J1.pollut_quality)
            # print(J2.pollut_quality)
            # print(J3.pollut_quality)
            # print(J4.pollut_quality)

        assert S1.buildup['test-pollutant'] == 25.000
        assert S2.buildup['test-pollutant'] == 25.000
        assert S3.buildup['test-pollutant'] == 25.000

        assert 9.99999 <= S1.conc_ponded['test-pollutant'] <= 10.00001
        assert 9.99999 <= S2.conc_ponded['test-pollutant'] <= 10.00001
        assert 9.99999 <= S3.conc_ponded['test-pollutant'] <= 10.00001

        assert S1.pollut_quality['test-pollutant'] == 0.0
        assert S2.pollut_quality['test-pollutant'] == 0.0
        assert S3.pollut_quality['test-pollutant'] == 0.0

        assert 0.0016769 <= S1.runoff_total_loading['test-pollutant'] <= 0.0016770
        assert 0.0012341 <= S2.runoff_total_loading['test-pollutant'] <= 0.0012342
        assert 0.00077987 <= S3.runoff_total_loading['test-pollutant'] <= 0.00077988

        assert C1.pollut_quality['test-pollutant'] == 0.0
        assert C2.pollut_quality['test-pollutant'] == 0.0
        assert C3.pollut_quality['test-pollutant'] == 0.0
        assert C4.pollut_quality['test-pollutant'] == 0.0
        
        assert 26.87 <= C1.total_loading['test-pollutant'] <= 26.88
        assert 46.64 <= C2.total_loading['test-pollutant'] <= 46.66
        assert 12.48 <= C3.total_loading['test-pollutant'] <= 12.50
        assert 59.12 <= C4.total_loading['test-pollutant'] <= 59.15

        assert J1.pollut_quality['test-pollutant'] == 0.0
        assert J2.pollut_quality['test-pollutant'] == 0.0
        assert J3.pollut_quality['test-pollutant'] == 0.0
        assert J4.pollut_quality['test-pollutant'] == 0.0
